#!/bin/sh
# -*- mode: landisk-sh; coding: utf-8; -*-
#
# landisk-update.sh
#
# $Id$
#

PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin
UPTGZ=$1
MODEL=`sed -n '/^firmware/s/^firmware	//p' /var/lib/model | sed -e 's|/|_|g'`
VERSION=`sed -n '/^version/s/^version	//p' /var/lib/model`

ROOTPATH='/mnt/hda5/.update'
VERFILE="${ROOTPATH}/for_${MODEL}_series"

UPFILE="/boot/update/update.tgz"
TARFILE="${ROOTPATH}/update-files.tar.gz"
BETAFILE="${ROOTPATH}/beta"
UPPERL="${ROOTPATH}/update.pl"
CONFDIR='/mnt/hda5/landisk/conffiles'
XMLCONFDIR='/mnt/hda5/xmlconf'
RMLIST="${ROOTPATH}/rmlist"
UBOOT="${ROOTPATH}/flash.uboot"
UBOOT_512MB="${ROOTPATH}/flash.uboot.512mb"
R8FIRM_DEF="${ROOTPATH}/r8firm.def"
MIGRATE_REPLICATION="${ROOTPATH}/landisk-update-migrate-replication.sh"
INSTALL_LOG="/mnt/hda5/factory_log/install.log"
FTP_CONFIG_UPDATE_CMD="${ROOTPATH}/ftp_config_update.pl"
PMUPDATE_CMD="${ROOTPATH}/pmupdate.pl"
ISSDISABLE_CMD="${ROOTPATH}/issdisable.pl"
TIMEMACHINE_UPDATE_CMD="${ROOTPATH}/timemachine_update.pl"
NETWORK_UPDATE_CMD="${ROOTPATH}/network_update.pl"
ACCESSLOG_UPDATE_CMD="${ROOTPATH}/accesslog_update.pl"
NEW_SHARE_CMD="${ROOTPATH}/new_share.pl"
UPDATE_CONFIG_SHARES_XML_CMD="${ROOTPATH}/update_config_shares_xml.py"
COMMONCONFIG_CMD="/mnt/hda5/bin/commonconfig"
IOBBCOM_CMD="/mnt/hda5/bin/iobbcom"
INIT_FLAG_FILE="/var/tmp/init"
SHARE_DIR="/mnt/sataraid1/share"
LIBEXEC_PRODUCT_DIR="/mnt/hda5/libexec/product"
INIT_CONF_DIR="/mnt/hda5/conf/product/init"
INIT_SHARE_FILE="$INIT_CONF_DIR/init_share"
INIT_SHARE_CMD="${ROOTPATH}/HDL_init_share"
MANUAL_DIR="/mnt/sataraid1/share/LAN DISK Manual"
MANUAL_PDF="${ROOTPATH}/README.txt"
TMP_UPTGZ="/mnt/sataraid1/update.tgz"
NEW_CONFIG_TGZ="${ROOTPATH}/new_config.tgz"
UPDATE_CONF_SHARE_CMD="${ROOTPATH}/update_conf_share.pl"
UPDATE_CONF_SMB_CMD="${ROOTPATH}/update_conf_smb.pl"

UPDATEVER=`sed 's/-.*$//' $VERFILE`
[ -f $BETAFILE ] && BETA=`head -n1 $BETAFILE`

start_led() {
   /mnt/hda5/bin/stscont -p skip -s process 
}

err_display() {
   /mnt/hda5/bin/stscont -s notice
   /mnt/hda5/bin/buzcont HDLE_BUZ_ERROR
}

# log出力
log_success() {
	local version=$1
	[ "x$version" = 'x' ] && return 1

	${UPPERL} --success="$version"
	return 0
}

# 終了処理
exit_select() {
	local result=$1

	err_display
	# output log
	${UPPERL} --fail
	rm -f $UPPERL

	exit $result
}

# 使用しなくなったもしくは移動したファイルディレクトリの消去
remove_unused_or_moved () {
	local rmdirlist="/etc/cups /etc/syslog-ng"

	for deldir in $rmdirlist; do
		if [ -d "$deldir" ]; then
			rm -rf $deldir
			if [ $? != 0 ]; then
				exit_select 8
			fi
		fi
	done
}

# 移動する
move_temp_updater() {
	local file="$1"
	local dest="$2"

	#
	# IE利用時にアップロードソースファイル名が渡されてしまった場合の対応。
	# 同一ディレクトリ内の".tgz"ファイルを対象とする。
	#
	if [ ! -f "$file" ]; then
		local dirpath=`dirname $file`

		for tgzfile in $dirpath/*.tgz; do
			if [ -f "$tgzfile" ]; then
				file="$tgzfile"
			fi
		done

		if [ ! -f "$file" ]; then
			return 1
		fi
	fi

	mv "$file" "$dest"
	return 0
}


# 移動する
move_updater() {
	local file="$1"

	#
	# IE利用時にアップロードソースファイル名が渡されてしまった場合の対応。
	# 同一ディレクトリ内の".tgz"ファイルを対象とする。
	#
	if [ ! -f "$file" ]; then
		local dirpath=`dirname $file`

		for tgzfile in $dirpath/*.tgz; do
			if [ -f "$tgzfile" ]; then
				file="$tgzfile"
			fi
		done

		if [ ! -f "$file" ]; then
			return 1
		fi
	fi

	mount -o remount,rw /boot
	mv "$file" /boot/update/update.tgz
	sync
	mount -o remount,ro /boot
	return 0
}


update_status_files()
{
	local OLD_DIR=/var/tmp/state
	local NEW_DIR=/var/tmp/conf/state
	local CATEGORY_LIST="system raid raid_vol active_repair backup \
		filesystem quickcopy  array_model"

	if [ ! -d "$OLD_DIR" ]; then
		return 0;
	fi

	if [ ! -d "$NEW_DIR" ]; then
		mkdir -p "$NEW_DIR"
	fi

	for category in $CATEGORY_LIST; do
		if [ -e "$OLD_DIR/$category" ]; then
			cp "$OLD_DIR/$category" "$NEW_DIR/${category}_status"
		fi
	done
}


update_iosupdate_config()
{
	local oldVerStr=`echo $VERSION | sed -e "s/\.//"`
	local iosupdate_enable_file="/mnt/hda5/conf/service/iosupdate_enable"

	if [ -z "$oldVerStr" -o -e $INIT_FLAG_FILE ]; then
		return
	fi

	if [ $oldVerStr -lt 110 ]; then
		touch $iosupdate_enable_file
	fi
}


update_iobbcom_config()
{

	local iobbcom_dir="/mnt/hda5/conf/iobbcom"
	local iobbcom_upnp_file="$iobbcom_dir/_upnp"

	mkdir -p $iobbcom_dir
	if [ -e $iobbcom_upnp_file ]; then
		return
	fi
	$IOBBCOM_CMD --set upnp 1
}


update_remoteaccess_config()
{
	local remoteaccess_port_file="/mnt/hda5/conf/remoteaccess/_port"

	if [ ! -e $remoteaccess_port_file ]; then
		$COMMONCONFIG_CMD remoteaccess --set port=50000
	fi

	$COMMONCONFIG_CMD --update remoteaccess
	$COMMONCONFIG_CMD --update remoteaccess share
}


update_snmptrap_config()
{
	local snmptrap_port_file="/mnt/hda5/conf/snmp/trap_port"

	if [ -e $snmptrap_port_file ]; then
		return
	fi
	$COMMONCONFIG_CMD snmp trap --set port=162
	$COMMONCONFIG_CMD snmp trap --set enable=0
}


add_new_users()
{
	adduser --quiet --system --no-create-home --home /var/lib/snmp snmp
}


force_remove_lock()
{
	local LOCK_DIR="/var/lock/udev"

	for lockfile in $LOCK_DIR/*.lock; do
		if [ -L "$lockfile" ]; then
			local original=`readlink $lockfile`

			case "$original" in
			*.[0-9]*)
				rm -f $lockfile
				;;
			*)
				;;
			esac
		fi
	done
}


make_directories()
{
	local dirlist="/mnt/hda5/etc/net /mnt/hda5/conf/dropbox \
			/mnt/hda5/xmlconf /mnt/hda5/packages \
			/mnt/hda5/packages/icons /mnt/hda5/packages/files \
			/mnt/hda5/packages/scripts"

	for dir in $dirlist; do
		mkdir -p $dir
	done
}


update_resetscript()
{
	mount -o remount,rw /boot
	cp ${ROOTPATH}/reset.sh /boot/landisk/reset.sh
	sync
	mount -o remount,ro /boot
}


update_proxy()
{
	local proxy_conf_file="$XMLCONFDIR/config-proxy.xml"
	local proxy_config='<proxy enable="false"><manual enable="false"><host/><port/></manual><pacfile enable="false"><url/></pacfile></proxy>'

	if [ ! -e $proxy_conf_file ]; then
		echo $proxy_config > $proxy_conf_file
	fi
	
}


update_packages()
{
	local packages_conf_file="$XMLCONFDIR/config-packages.xml"
	local packages_config='<packages><config><serverinfo><protocol>http</protocol><hostname>iosupdate.iodata.jp</hostname><path>lib/auto/hdl-xr/packages/2</path><infofile>HDL-XR_packages.xml</infofile></serverinfo><localinfo><pathlist><path>/mnt/usb1/hdl-xr</path><path>/mnt/usb2/hdl-xr</path></pathlist></localinfo></config></packages>'

	if [ ! -e $packages_conf_file ]; then
		echo $packages_config > $packages_conf_file
	fi

	if grep -e "<path>lib/auto/hdl-xr/packages</path>" \
			$packages_conf_file > /dev/null 2> /dev/null; then
		sed -e "s:\(<path>lib/auto/hdl-xr/packages\)\(</path>\):\1/2\2:" $packages_conf_file > ${packages_conf_file}.$$
		mv $packages_conf_file.$$ ${packages_conf_file}
	fi
}


update_shares()
{
	local shares_conf_file="$XMLCONFDIR/config-shares.xml"
	local shares_config='<shares><sharelist><share><name>disk1</name><path>/mnt/sataraid1/share/disk1</path><shareservice><amazons3 enable="false"/></shareservice></share><share><name>usb1</name><path>/mnt/usb1</path><shareservice><amazons3 enable="false"/></shareservice></share><share><name>usb2</name><path>/mnt/usb2</path><shareservice><amazons3 enable="false"/></shareservice></share><share><name>esata1</name><path>/mnt/esata1</path><shareservice><amazons3 enable="false"/></shareservice></share></sharelist></shares>'

	if [ ! -e $shares_conf_file ]; then
		echo $shares_config > $shares_conf_file
	fi
	
	# 既存の共有をconfig-shares.xmlに追加する。
	# fal_sharesは変更の際に既存共有かをチェックしているので、
	# ここで追加しないとそれら共有はfal_sharesから変更できなくなる。
	${UPDATE_CONFIG_SHARES_XML_CMD}
}


update_uboot()
{
	local uboot_img=$UBOOT

	local memsize=`cat /proc/meminfo | grep MemTotal | \
			sed -e "s/MemTotal:[^0-9]*\([0-9][0-9]*\).*/\1/"`

	if [ $memsize -gt 500000 ]; then
		uboot_img=$UBOOT_512MB
	fi

	# uBootが古い場合更新する
	if ! cmp $uboot_img /dev/mtdblock0 > /dev/null 2> /dev/null; then
		dd if=$uboot_img of=/dev/mtdblock0 bs=64k || exit_select 1
	fi
}


########################
# start
start_led

# Remount for ensuring that /mnt/sataraid1 is writable.
mount -o remount,rw,noatime /
move_temp_updater "$UPTGZ" "$TMP_UPTGZ"

# wait display cgi
sleep 20

## perlでアップデート
#${UPPERL} --update
#if [ $? != 0 ]; then
#    exit_select 3
#fi

# remount rw /
mount -o remount,rw,noatime /

# サービスの停止 - syslog-ng, R8contd は止めない
/mnt/hda5/bin/servicectrl --set stop
for i in HDLFind.sh cron lsync lsyncd-log-filter slpd digion-lltd-eth1 digion-lltd-eth0 digion-dpws udev dbus mdadm apache2 networking
do
    /etc/init.d/$i stop
done

# 必要であれば
#killall busybox			# telnetd
killall landiskAdminDaemon
killall HDL_iosupdate


remove_unused_or_moved

# tarで展開
mount -o remount,rw /boot
tar xpzf ${TARFILE} -C / || exit_select 1

mount -o remount,ro /boot

# インストール時にver.1.20～1.30だったものはreset.shを更新する
update_resetscript

force_remove_lock

# ユーザー追加。
add_new_users

# ステータスファイルを更新
update_status_files

make_directories
update_iosupdate_config
# remoteaccessの前にiobbcomの設定更新を行う必要あり
update_iobbcom_config
update_remoteaccess_config
update_snmptrap_config
update_proxy
update_packages
update_shares

update_uboot

# R8展開
/usr/local/bin/R8fwrite -p iodata -d ${R8FIRM_DEF} || exit_select 1

# レプリケーション設定ファイルのアップデート
if [ -x ${MIGRATE_REPLICATION} ]; then
	${MIGRATE_REPLICATION}
fi

# FTP設定ファイルのアップデート
if [ -x ${FTP_CONFIG_UPDATE_CMD} ]; then
	${FTP_CONFIG_UPDATE_CMD}
fi

${PMUPDATE_CMD}
${ISSDISABLE_CMD}
${TIMEMACHINE_UPDATE_CMD}
${NETWORK_UPDATE_CMD}
${ACCESSLOG_UPDATE_CMD}

if [ -e "$MANUAL_DIR" -a ! -d "$MANUAL_DIR" ]; then
	rm -f "$MANUAL_DIR"
	/usr/local/bin/HDL_share_set --del --name="LAN DISK Manual"
fi

if [ -d $SHARE_DIR ]; then
	${NEW_SHARE_CMD}
	cp -p $MANUAL_PDF "$MANUAL_DIR"
	pdfname=`basename $MANUAL_PDF`
	chmod 766 "$MANUAL_DIR/$pdfname"
	chown nobody.nogroup "$MANUAL_DIR/$pdfname"
else
	mkdir -p $INIT_CONF_DIR
	mkdir -p $LIBEXEC_PRODUCT_DIR
	touch $INIT_SHARE_FILE
	cp -p $INIT_SHARE_CMD $LIBEXEC_PRODUCT_DIR
	cp $MANUAL_PDF $LIBEXEC_PRODUCT_DIR
fi

if [ -e $INIT_FLAG_FILE -o x`basename $UPTGZ` = x"update.tgz" ]; then
	tar xpzf ${NEW_CONFIG_TGZ} -C / || exit_select 1
else
	${UPDATE_CONF_SHARE_CMD}
fi

# config-samba_conf の更新
if [ -x "${UPDATE_CONF_SMB_CMD}" ]; then
	"${UPDATE_CONF_SMB_CMD}"
fi

# remount rw /usr/local/bin/HDL_share_set
mount -o remount,rw,noatime /

# 不要となったファイルの削除
RAID_FILE_EXIST=""

for file in /mnt/hda5/conf/raid/debug_*; do
	if [ -f $file ]; then 
		RAID_FILE_EXIST="yes"
		rm -f $file
	fi
done

if [ x"$RAID_FILE_EXIST" = x"yes" ]; then
	echo "production=type1($UPDATEVER:`LANG=C date`)" >> $INSTALL_LOG
fi


# rmlistによる削除
if [ -e ${RMLIST} ]; then
    cat ${RMLIST} | xargs rm -f
fi

# ファイルを削除する
rm -f $TARFILE $BETAFILE $VERFILE $UBOOT $UBOOT_512MB $FTP_CONFIG_UPDATE_CMD \
	$PMUPDATE_CMD $ISSDISABLE_CMD $TIMEMACHINE_UPDATE_CMD \
	$NETWORK_UPDATE_CMD $ACCESSLOG_UPDATE_CMD $NEW_SHARE_CMD $MANUAL_PDF \
	$INIT_SHARE_CMD $UPDATE_CONFIG_SHARES_XML_CMD \
	$NEW_CONFIG_TGZ

echo "$UPDATEVER" > /etc/version
if [ "$BETA" != "" ]; then
	echo $BETA > /etc/beta
else
	rm -f /etc/beta
fi

# 移動する
move_updater "$TMP_UPTGZ"

# log処理
log_success "$UPDATEVER"
rm -f $UPPERL

/usr/local/bin/HDL_shutdown --force --reboot

exit 0
