#!/usr/bin/perl

use strict;
use warnings;

use HDLi_group;

use constant {
	RETRY_COUNT => 3,
        GROUP_FILE  => '/etc/group',
};


sub __get_etc_group_setting()
{
	my $fh;

	if (!open($fh, GROUP_FILE)) {
		return undef;
	}

	my $setting = {};
	while ( <$fh> ) {
		chomp;

		my ($name, undef, undef, $member ) = split( ':', $_ );

		$setting->{$name} = {};
		foreach my $uname (split(',', $member )) {
			$setting->{$name}->{$uname} = 1;
		}
	}
	close ( $fh );
	return $setting;
}


sub __check_group_setting($)
{
	my $hdl_setting = shift;
	my $etc_setting = __get_etc_group_setting();

	foreach my $gname (keys %{$hdl_setting}) {
		if (!exists($etc_setting->{$gname})) {
			return 0;
		}
		foreach my $uname (keys %{$hdl_setting->{$gname}->{'member'}}) {
			if (!exists($etc_setting->{$gname}->{$uname})) {
				return 0;
			}
		}
		foreach my $uname (keys %{$etc_setting->{$gname}}) {
			if (!exists($hdl_setting->{$gname}->{'member'}->{$uname})) {
				return 0;
			}
		}
	}
	return 1;
}



sub __update_group_setting($)
{
	my $hdl_setting = shift;
	my $etc_setting = __get_etc_group_setting();

	foreach my $gname (keys %{$hdl_setting}) {
		if (!exists($etc_setting->{$gname})) {
			my $gid = $hdl_setting->{$gname}->{'gid'};

			`/usr/sbin/groupadd -g $gid $gname`;
			$etc_setting->{$gname} = {};
		}
		foreach my $uname (keys %{$hdl_setting->{$gname}->{'member'}}) {
			if (!exists($etc_setting->{$gname}->{$uname})) {
				`/usr/bin/gpasswd -a $uname $gname`;
			}
		}
		foreach my $uname (keys %{$etc_setting->{$gname}}) {
			if (!exists($hdl_setting->{$gname}->{'member'}->{$uname})) {
				`/usr/bin/gpasswd -d $uname $gname`;
			}
		}
	}
}


################################################
# MAIN
################################################

my $groupobj = HDLi_group->new();

if (!defined($groupobj)) {
	print "Error : can't new module\n";
	exit 1;
}

my $hdl_group_setting = $groupobj->get();
my $try_count;

for ($try_count = 0; $try_count < RETRY_COUNT; $try_count++ ) {
	if (__check_group_setting($hdl_group_setting)) {
		exit 0;
	}
	__update_group_setting($hdl_group_setting);
}

undef $groupobj;

exit 1;
