#!/bin/sh
#
# reset.sh: システム初期化を行う
# Copyright (C) 2009 I-O DATA DEVICE, INC.  All rights reserved.
#

MODEL="HDL-XR%s"			# モデル名(printf 書式: %d は容量)
USAGE="HDL-XR"				# デフォルトの名前

DISK_MD1="/dev/md1"
DISK_MD2="/dev/md2"
DISK_MD5="/dev/md5"
DISK_MD6="/dev/md6"
DISK_MD13="/dev/md13"
DIR_MNT="/mnt"
MNT_MD1="$DIR_MNT/md1"
MNT_MD2="$DIR_MNT/md2"
MNT_MD5="$DIR_MNT/md5"
MNT_MD6="$DIR_MNT/md6"

FOLDER_RESETSRC="${MNT_MD1}/landisk"
FOLDER_FACTLOG="${MNT_MD5}/factory_log"
FOLDER_DEVUUID="${MNT_MD5}/conf/init"

TWOHD=""

# sector_disk2lbaにてNOIDEMAオプションの判定があるため
FLAG_NOIDEMA=0

# for encrypt
ENC_DEV_ROOT="/dev/mapper"
ENC_DEV_MD13="md13_enc"
#CRYPT_PROG="/sbin/cryptsetup.sh"

. /etc/rc.d/diskmount.sh
. /etc/rc.d/diskprobe.sh
. /etc/rc.d/mddetect.sh
. /etc/rc.d/sector.sh
. /etc/rc.d/disklabel.sh

###############################################################################
# 実行可能環境かの確認
###############################################################################
__resetscript_is_executable() {
    FUNC="__resetscript_is_executable";

    # ディスクの接続状態を調べる
    if ! DISK_SATA1=`diskprobe_probe_physical_disk 'SATA1'`; then
	# ディスクの接続状態が分からない
	echo "*** $FUNC: ERROR: PROBE FAILED sata1 ***"
    fi
    if ! DISK_SATA2=`diskprobe_probe_physical_disk 'SATA2'`; then
	# ディスクの接続状態が分からない
	echo "*** $FUNC: ERROR: PROBE FAILED sata2 ***"
    fi
    if ! DISK_SATA3=`diskprobe_probe_physical_disk 'SATA3'`; then
	# ディスクの接続状態が分からない
	echo "*** $FUNC: ERROR: PROBE FAILED sata3 ***"
    fi
    if ! DISK_SATA4=`diskprobe_probe_physical_disk 'SATA4'`; then
	# ディスクの接続状態が分からない
	echo "*** $FUNC: ERROR: PROBE FAILED sata4 ***"
    fi

    # 内蔵ディスクが 4 台接続されているか
    if [ "$DISK_SATA1" != '' -a "$DISK_SATA2" != '' \
	-a "$DISK_SATA3" = '' -a "$DISK_SATA4" = '' ]; then
	# /2HD モード
	TWOHD=1
    elif [ "$DISK_SATA1" = '' -o "$DISK_SATA2" = '' \
	    -o "$DISK_SATA3" = '' -o "$DISK_SATA4" = '' ]; then
	# ディスクが 4 台ない
	echo "*** $FUNC: ERROR: PROBED DISK IS LESS THAN 4 ***"
	return 1
    fi
    
    # 2発モードで4発接続されていたとき
    # disklabelのmodelを参照し、2発モードで初期化する
    if [ "$TWOHD" = '' ]; then
	disklabel_file_read $DISK_SATA1 model | grep "2D" 2>/dev/null
	if [ $? -eq 0 ]; then
	    TWOHD=1
	fi
    fi

    # 開発検証用環境か

    local SIZE_PART6=0

    [ "$SIZE_PART6" -eq 0 ] &&
    mddetect_is_system_disk 'SATA1' &&
    let SIZE_PART6=`cat /sys/block/${DISK_SATA1}/${DISK_SATA1}6/size 2>/dev/null`/2

    [ "$SIZE_PART6" -eq 0 ] &&
    mddetect_is_system_disk 'SATA2' &&
    let SIZE_PART6=`cat /sys/block/${DISK_SATA2}/${DISK_SATA2}6/size 2>/dev/null`/2

    if [ "$TWOHD" = '' ]; then
	[ "$SIZE_PART6" -eq 0 ] &&
	mddetect_is_system_disk 'SATA3' &&
	let SIZE_PART6=`cat /sys/block/${DISK_SATA3}/${DISK_SATA3}6/size 2>/dev/null`/2

	[ "$SIZE_PART6" -eq 0 ] &&
	mddetect_is_system_disk 'SATA4' &&
	let SIZE_PART6=`cat /sys/block/${DISK_SATA4}/${DISK_SATA4}6/size 2>/dev/null`/2
    fi

    if [ "$SIZE_PART6" -gt 5875000 ]; then
	# 通常用
	FLAG_DEBUG=0
    else
	# 開発検証用
	FLAG_DEBUG=1
    fi

    # md1 のマウント
    if ! diskmount_mount rw "$DISK_MD1" "$MNT_MD1"; then
	# マウントできない
	echo "*** $FUNC: ERROR: FAILED TO MOUNT md1 ***"
	return 2
    fi

    # 実行 OK
    return 0
}



###############################################################################
# __resetscript_reset_partition_disklabel_raid 用サブルーチン
#
# __resetscript_reset_a_partition SATA1 sda
#
# $1: ポート名
# $2: デバイス名
#
###############################################################################
__resetscript_reset_a_partition () {
    local port="$1" dev="$2"
    local FUNC='__resetscript_reset_a_partition'

    mddetect_is_system_disk "$port" && return 0

    # 起動用でない -> リビルドする
    echo "*** $FUNC: $port IS NOT BOOTABLE ***"
    echo "*** $FUNC: REBUILD $port ***"
    
    # パーティションテーブルの作成
    if ! __resetscript_write_partition_table "$dev" ; then
	echo
	echo "*** $FUNC: ERROR: FAILED TO WRITE PARTITION TABLE TO $dev ***"
	return 1
    fi
    
    # ディスクラベルの書き込み
    if ! __resetscript_write_disklabel_full "$dev"; then
	echo
	echo "*** $FUNC: ERROR: FAILED TO WRITE DISKLABEL TO $dev ***"
	return 1
    fi
    
	# SWAP の作成
    if ! /sbin/mkswap "/dev/${dev}3" 1>/dev/null 2>/dev/null; then
	echo "*** $FUNC: ERROR: FAILED TO FORMAT SWAP PARTITION DISK=${dev}3 ***"
	return 1
    fi
    
    # RAID に hot add する
    local p
    for p in 1 2 5; do
	local DISK_MD=""
	
	case $p in
	    1) DISK_MD="$DISK_MD1";;
	    2) DISK_MD="$DISK_MD2";;
	    5) DISK_MD="$DISK_MD5";;
	    *) DISK_MD="";;
	esac
	
	/sbin/mdadm \
	    --zero-superblock \
	    --force \
	    "/dev/$dev$p" \
	    1>/dev/null 2>/dev/null
	dd \
	    if=/dev/zero \
	    of="/dev/$dev$p" \
	    bs=1k \
	    count=4 \
	    1>/dev/null 2>/dev/null
	/sbin/mdadm \
	    "$DISK_MD" \
	    --force \
	    --add "/dev/$dev$p" \
	    1>/dev/null 2>/dev/null
    done
    
    echo "*** $FUNC: REBUILD $port OK ***"
    return 0
}


###############################################################################
# RAID を組めていないディスクを初期化する
###############################################################################
__resetscript_reset_partition_disklabel_raid() {
    local FUNC="__resetscript_reset_partition_disklabel_raid"

    if [ "$TWOHD" = '' ]; then
	__resetscript_reset_a_partition 'SATA1' "$DISK_SATA1" || return 1
	__resetscript_reset_a_partition 'SATA4' "$DISK_SATA4" || return 2
	__resetscript_reset_a_partition 'SATA2' "$DISK_SATA2" || return 3
	__resetscript_reset_a_partition 'SATA3' "$DISK_SATA3" || return 4
    else
	__resetscript_reset_a_partition 'SATA1' "$DISK_SATA1" || return 1
	__resetscript_reset_a_partition 'SATA2' "$DISK_SATA2" || return 2
    fi

    return 0
}


###############################################################################
# パーティションテーブルを書き込む
#
# $1: 書き込み先デバイス
###############################################################################
__resetscript_write_partition_table() {
    local DISK="$1"

    local lastlba=`sector_disk2lba ${MODELDISK}`
    [ "$lastlba" = "error" ] && return 1

    /bin/dd if=/dev/zero of="/dev/${DISK}" bs=512 count=2 \
	1>/dev/null 2>/dev/null || return 1

    if [ ${FLAG_DEBUG} -eq 1 ]; then
        # 開発検証用
	lastlba=20000000
    else
        # ディスク末尾のパーティションテーブル分を差し引く
#        let lastlba=$lastlba-34
	# 4KB物理セクタ(論理512B)HDDへの対応前後で第6パーティション(データパーティション)サイズを合わせるため
	let lastlba=$lastlba-35
    fi

    /sbin/parted -s /dev/$DISK \
        mktable gpt \
        unit s \
        mkpartfs primary fat32 40 1000006 \
        mkpart primary ext2 1000008 3000007 \
        mkpart primary linux-swap 3000008 6999999 \
        mkpartfs primary ext2 7000000 7249999 \
        mkpart primary ext2 7250000 8249999 \
        mkpart primary ext2 8250000 $lastlba || return 1

    return 0
}


###############################################################################
# factory_log の退避
###############################################################################
__resetscript_save_factory_log() {
    local PWD_OLD=`pwd`

    # 退避済が存在していたら初期化中にshutdownされた可能性あり。
    # すでに退避済なので処理を抜ける。
    if [ -e "$FOLDER_RESETSRC/factory_log.tar.gz" ]; then
	# factory_log.tar.gzがすでにある（すでに一度は初期化処理を行った）
	diskmount_umount "$MNT_MD5"
	return 0
    fi

    # md5 のマウント
    diskmount_mount ro "$DISK_MD5" "$MNT_MD5" || return 1

    if [ ! -d "$FOLDER_FACTLOG" ]; then
	# factory_log が存在しない
	diskmount_umount "$MNT_MD5"

	return 1
    fi

    # 前のが存在したら消す．エラーは無視
    rm -f "$FOLDER_RESETSRC/factory_log.tar.gz"

    # 退避
    cd "$MNT_MD5"
    if ! tar cf - "factory_log" | gzip -c9 > "$FOLDER_RESETSRC/factory_log.tar.gz"; then
	# factory_log を保存できない
	cd "$PWD_OLD"
	diskmount_umount "$MNT_MD5"
	return 2
    fi

    # 後始末
    cd "$PWD_OLD"
    diskmount_umount "$MNT_MD5"

    return 0
}


###############################################################################
# device_uuid の退避
###############################################################################
__resetscript_save_device_uuid() {
    local PWD_OLD=`pwd`

    if [ -e "${FOLDER_RESETSRC}/device_uuid" ]; then
	# 初期化中に shutdown された可能性あり
	# すでに退避済なので処理を抜ける。
	diskmount_umount "${MNT_MD5}"
	return 0
    fi

    # md5 のマウント
    diskmount_mount ro "${DISK_MD5}" "${MNT_MD5}" || return 1

    if [ ! -d "${FOLDER_DEVUUID}" ]; then
	# コピー元がが存在しない
	diskmount_umount "${MNT_MD5}"

	return 1
    fi

    # 前のが存在したら消す
    if [ -e "${FOLDER_RESETSRC}/device_uuid" ]; then
	/bin/rm -f "${FOLDER_RESETSRC}/device_uuid"
    fi

    # 退避
    if ! /bin/cp -a "${FOLDER_DEVUUID}/device_uuid" "${FOLDER_RESETSRC}/device_uuid"; then
	# 保存できない
	diskmount_umount "${MNT_MD5}"
	return 2
    fi

    # 後始末
    diskmount_umount "${MNT_MD5}"

    return 0
}


###############################################################################
# ディスクのLBAよりモデル名を決定する
###############################################################################
__resetscript_get_lba_mindisk() {
    local DISK=""
    local SET=$1
    local LBA=0
    local MINLBA=`cat "/sys/block/$DISK_SATA1/size"`
    local MINDISK="$DISK_SATA1"
    local SEC=""

    if [ "$TWOHD" = '' ]; then
	for DISK in "$DISK_SATA2" "$DISK_SATA3" "$DISK_SATA4"
	do
	    LBA=`cat "/sys/block/$DISK/size"`
	    if [ "$LBA" -lt "$MINLBA" ]; then
		MINLBA=$LBA
		MINDISK=$DISK
	    fi
	done
    else
	LBA=`cat "/sys/block/$DISK_SATA2/size"`
	if [ "$LBA" -lt "$MINLBA" ]; then
	    MINLBA=$LBA
	    MINDISK=$DISK_SATA2
	fi
    fi

    # 250GB以上のドライブか確認
    SEC=`sector_disk2lba "$MINDISK"`
    [ "$SEC" = 'error' ] && return 1


    if [ $SET -eq 0 ]; then
        echo "$MINDISK"
    else
        echo "$MINLBA"
    fi

}


###############################################################################
# disklabel_identフィールドの生成
###############################################################################
__resetscript_make_ident() {
    local DISK=$1
    local SUM=""

    [ "$DISK" = '' ] && return 1
    [ ! -x /usr/local/bin/mv_identify ] && return 1

    SUM=`/usr/local/bin/mv_identify -b /dev/$DISK | md5sum | awk '{print $1}'`
    [ "$SUM" = '' ] && return 1

    # fail mv_identify (blank)
    [ "$SUM" = 'd41d8cd98f00b204e9800998ecf8427e' ] && return 1

    echo "$SUM"
    return 0
}


###############################################################################
# ディスクラベルの更新
###############################################################################
__resetscript_write_disklabel() {
    local FUNC="__resetscript_write_disklabel"
    local DISK="$1"
    local IDENT="$(__resetscript_make_ident ${DISK})"
    # local MODEL_FULL="$(sector_disk2model ${MODELDISK} ${TWOHD})"
    local LBA=`__resetscript_lastlba_for_labelelement`

    local HDNUM=""

    if [ "$TWOHD" = '' ]; then
        HDNUM=4
    else
        HDNUM=2
    fi

	# RTSモデルならディスクラベルのモデル名もRTSモデルにする
	if [ "$RTSFLAG" = 'ON' ]; then
		local MODEL_FULL=`/etc/rc.d/labelelement.sh m $LBA $HDNUM r`
	else
		local MODEL_FULL=`/etc/rc.d/labelelement.sh m $LBA $HDNUM`
	fi

    # 引数が不正
    [ "$DISK" = '' ] && return 1

    # この時点で HDL-GT 用システムとしてのディスクラベルは設定済みであるので
    # RAID 構成部分のみ初期化する


    # mindisk(rev.0002)
    # raidmode(rev.0002)
    if [ "$TWOHD" = '' ]; then
	disklabel_file_write "$DISK" 'mindisk' '2'
	disklabel_file_write "$DISK" 'raidmode' 'raid6'
    else
	disklabel_file_write "$DISK" 'mindisk' '1'
	disklabel_file_write "$DISK" 'raidmode' 'raid1'
    fi

    # model名も更新する
    disklabel_file_write "$DISK" 'model' "$MODEL_FULL"

    # ident(rev.0002)
    disklabel_file_write "$DISK" 'ident' "$IDENT"

    # md5sum
    disklabel_write_md5sum "$DISK"
    if [ $? -ne 0 ]; then
        echo "*** $FUNC : FAILED TO CHECK $DISK DISKLABEL MD5SUM ***"
        return 2
    fi

    return 0
}


###############################################################################
# ディスクラベルの更新(フル)
###############################################################################
__resetscript_write_disklabel_full() {
    local FUNC="__resetscript_write_disklabel_full"
    local DISK="$1"
    # local MODEL_FULL="$(sector_disk2model ${MODELDISK} ${TWOHD})"
    local LBA=`__resetscript_lastlba_for_labelelement`
    
    local HDNUM=""

    if [ "$TWOHD" = '' ]; then
        HDNUM=4
    else
        HDNUM=2
    fi

	# RTSモデルならディスクラベルのモデル名もRTSモデルにする
	if [ "$RTSFLAG" = 'ON' ]; then
		local MODEL_FULL=`/etc/rc.d/labelelement.sh m $LBA $HDNUM r`
	else
		local MODEL_FULL=`/etc/rc.d/labelelement.sh m $LBA $HDNUM`
	fi

    local IDENT="$(__resetscript_make_ident ${DISK})"

    # magic
    disklabel_file_write "$DISK" 'magic' 'HDLG'

    # version
    disklabel_file_write "$DISK" 'version' '2'

    # usage
    disklabel_file_write "$DISK" 'usage' 'RAID'

    # mindisk(rev.0002)
    # raidmode(rev.0002)
    if [ "$TWOHD" = '' ]; then
	disklabel_file_write "$DISK" 'mindisk' '2'
	disklabel_file_write "$DISK" 'raidmode' 'raid6'
    else
	disklabel_file_write "$DISK" 'mindisk' '1'
	disklabel_file_write "$DISK" 'raidmode' 'raid1'
    fi

    # model
    disklabel_file_write "$DISK" 'model' "$MODEL_FULL"

    # port
    disklabel_file_write "$DISK" 'port' ''

    # errorcount(rev.0002)
    disklabel_file_write "$DISK" 'errorcount' '0'

    # ident(rev.0002)
    disklabel_file_write "$DISK" 'ident' "$IDENT"

    # name
    disklabel_file_write "$DISK" 'name' "$USAGE"

    # md5sum
    disklabel_write_md5sum "$DISK"
    if [ $? -ne 0 ]; then
        echo "*** $FUNC : FAILED TO CHECK $DISK DISKLABEL MD5SUM ***"
        return 1
    fi

    return 0
}

###############################################################################
# パーティションから相応のlastlbaを求める (* 実際のディスクサイズは無関係)
# ディスクの入れ替えで大容量のHDDに小さなパーティションが書かれる場合がある
# このときHDDサイズからモデル名を決めると大容量帯のモデル名になってしまう
# そのためモデル名生成用にパーティションサイズ相応のlastlbaを取得する関数
###############################################################################
__resetscript_lastlba_for_labelelement() {
	local minlba=""
	for DISK in "${DISK_SATA1}" "${DISK_SATA2}" "${DISK_SATA3}" "${DISK_SATA4}"
	do
		local start=0
		local size=0
		local start_file="/sys/block/${DISK}/${DISK}6/start"
		local size_file="/sys/block/${DISK}/${DISK}6/size"
		if [ -f "${start_file}" -a -f "${size_file}" ]; then
			start=`cat /sys/block/${DISK}/${DISK}6/start`
			size=`cat /sys/block/${DISK}/${DISK}6/size`
		else
			continue
		fi
		local lba=`expr $start + $size`
		if [ "${lba}" = "" ]; then
			continue
		elif [ "${minlba}" = "" ]; then
			minlba=${lba}
		elif [ "${lba}" -lt "${minlba}" ]; then
			minlba=${lba}
		fi
	done

	if [ "${minlba}" = "" ]; then
		# 250GBの値
		echo 488397168
	else
		minlba=`expr ${minlba} + 500000`
		echo ${minlba}
	fi
}

###############################################################################
# RTC アラーム設定を無効にする
###############################################################################
__resetscript_reset_alarm() {
    local FUNC="__resetscript_reset_alarm"

    if [ ! -x /usr/local/sbin/alarm_off ]; then
	# コマンドがない(通常ここにはこない)
	echo "*** ${FUNC}: ERROR: COMMAND alarm_off NOT FOUND ***"
	return 1
    fi

    for ALARM in 'A' 'B'; do
	/usr/local/sbin/alarm_off "${ALARM}"
	if [ ${?} -ne 0 ]; then
	    # アラーム解除失敗
	    echo "*** ${FUNC}: ERROR: FAILED TO RESET ALARM ${ALARM} ***"
	    return 1
	fi
    done

    return 0
}


###############################################################################
# md6 を 0 クリアする
###############################################################################
__resetscript_sanitize_md6() {
    local FUNC="__resetscript_sanitize_md6"

    # RAID1 で RAID 構築(md6)
    echo "*** $FUNC: SANITIZE DISK ***"

    ## スーパーブロックをクリアする
    for DISK in "$DISK_SATA1" "$DISK_SATA2" "$DISK_SATA3" "$DISK_SATA4"; do
	[ "$TWOHD" != '' -a "$DISK" = '' ] && break
	/sbin/mdadm \
	    --zero-superblock \
	    --force \
	    "/dev/${DISK}6" \
	    1>/dev/null 2>/dev/null
	/bin/dd \
	    if=/dev/zero \
	    of="/dev/${DISK}6" \
	    bs=1M \
	    count=4 \
	    1>/dev/null 2>/dev/null
    done

    ## RAID1 を組む
    if [ "$TWOHD" = '' ]; then
	if ! /sbin/mdadm \
	    --create "$DISK_MD6" \
	    --force \
	    --assume-clean \
	    --run \
	    --level=raid1 \
	    --raid-devices=4 \
	    "/dev/${DISK_SATA1}6" \
	    "/dev/${DISK_SATA2}6" \
	    "/dev/${DISK_SATA3}6" \
	    "/dev/${DISK_SATA4}6" \
	    1>/dev/null 2>/dev/null; then
	    echo "*** ${FUNC}: ERROR: FAILED TO CREATE RAID ${DISK_MD6} ***"
	    return 1
	fi
    else
	if ! /sbin/mdadm \
	    --create "$DISK_MD6" \
	    --force \
	    --assume-clean \
	    --run \
	    --level=raid1 \
	    --raid-devices=2 \
	    "/dev/${DISK_SATA1}6" \
	    "/dev/${DISK_SATA2}6" \
	    1>/dev/null 2>/dev/null; then
	    echo "*** ${FUNC}: ERROR: FAILED TO CREATE RAID ${DISK_MD6} ***"
	    return 1
	fi
    fi

    # 0 クリアする
    /bin/dd \
	if=/dev/zero \
	of="${DISK_MD6}" \
	bs=1M \
	1>/dev/null 2>/dev/null

    # RAID1 を解く
    sync; sync; sync
    if ! /sbin/mdadm \
	--stop ${DISK_MD6} \
	1>/dev/null 2>/dev/null; then
	echo "*** ${FUNC}: ERROR: FAILED TO STOP RAID ${DISK_MD6} ***"
	return 1
    fi

    # スーパーブロックをクリアする
    for DISK in "$DISK_SATA1" "$DISK_SATA2" "$DISK_SATA3" "$DISK_SATA4"; do
	[ "$TWOHD" != '' -a "$DISK" = '' ] && break
	/sbin/mdadm \
	    --zero-superblock \
	    --force \
	    "/dev/${DISK}6" \
	    1>/dev/null 2>/dev/null
	/bin/dd \
	    if=/dev/zero \
	    of="/dev/${DISK}6" \
	    bs=1M \
	    count=4 \
	    1>/dev/null 2>/dev/null
    done

    return 0
}


###############################################################################
# md1 を初期化する
###############################################################################
__resetscript_reset_md1() {
    local FUNC="__resetscript_reset_md1"
    local FILE="$FOLDER_RESETSRC/md1.tgz"

    # 上書き用データはあるか
    if [ ! -e "$FILE" ]; then
	# データがない
	echo "*** $FUNC: ERROR: NO DATA ***"
	return 1
    fi

    # 上書き
    if ! tar zxf "$FILE" -C "$MNT_MD1"; then
	# 上書きできない
	echo "*** $FUNC: ERROR: FAILED TO COPY FILES ***"
	return 2
    fi

    return 0
}


###############################################################################
# md2 を初期化する
###############################################################################
__resetscript_reset_md2() {
    local FUNC="__resetscript_reset_md2"
    local FILE="$FOLDER_RESETSRC/md2.tgz"

    # 上書き用データはあるか
    if [ ! -e "$FILE" ]; then
	# データがない
	echo "*** $FUNC: ERROR: NO DATA ***"
	return 1
    fi

    # フォーマット
    if ! /sbin/mke2fs -j -m1 -q -F "$DISK_MD2"; then
	echo "*** $FUNC: ERROR: FAILED TO FORMAT VOLUME ***"
	return 2
    fi

    # マウント
    if ! diskmount_mount rw "$DISK_MD2" "$MNT_MD2"; then
	# マウントできない
	echo "*** $FUNC: ERROR: FAILED TO MOUNT VOLUME ***"
	return 3
    fi

    # コピー
    if ! tar zxf "$FILE" -C "$MNT_MD2"; then
	# コピーできない
	diskmount_umount "$MNT_MD2"
	echo "*** $FUNC: ERROR: FAILED TO COPY FILES ***"
	return 4
    fi

    # 後始末
    diskmount_umount "$MNT_MD2"

    return 0
}


###############################################################################
# md5 を初期化する
###############################################################################
__resetscript_reset_md5() {
    local FUNC="__resetscript_reset_md5"
    local FILE="$FOLDER_RESETSRC/md5.tgz"

    # 上書き用データはあるか
    if [ ! -e "$FILE" ]; then
	# データがない
	echo "*** $FUNC: ERROR: NO DATA ***"
	return 1
    fi

    # フォーマット
    if ! /sbin/mke2fs -q -F -j \
	-b 1024 \
	-N 100000 \
	-O dir_index,filetype,sparse_super \
	"$DISK_MD5" \
	1>/dev/null 2>/dev/null; then
	echo "*** $FUNC: ERROR: FAILED TO FORMAT VOLUME ***"
	return 2
    fi

    # マウント
    if ! diskmount_mount rw "$DISK_MD5" "$MNT_MD5"; then
	# マウントできない
	echo "*** $FUNC: ERROR: FAILED TO MOUNT VOLUME ***"
	return 3
    fi

    # コピー
    if ! tar zxf "${FILE}" -C "${MNT_MD5}"; then
	# コピーできない
	diskmount_umount "$MNT_MD5"
	echo "*** $FUNC: ERROR: FAILED TO COPY FILES ***"
	return 4
    fi

    # 2発モード時に必要なRAID情報を設定する
    local SLOT3="sata3_ignore"
    local SLOT4="sata4_ignore"
    local MD6MODE="md6_mode"
    local SATARAID1="sataraid1_mode"
    local MODE=""
    local RESULT=0

    if [ "$TWOHD" != '' ]; then

	for MODE in "$SLOT3" "$SLOT4"; do
	    touch "$MNT_MD5/conf/port/$MODE"
	    if [ $? -ne 0 ]; then
		echo "*** $FUNC: ERROR: FAILED TO TOUCH FILE(S) FOR 2D_MODE ***"
		RESULT=1
	    fi
	done

	for MODE in "$MD6MODE" "$SATARAID1"; do
	    echo "raid1" >  "$MNT_MD5/conf/raid/$MODE"
	    if [ $? -ne 0 ]; then
		echo "*** $FUNC: ERROR: FAILED TO TOUCH FILE(S) FOR 2D_MODE ***"
		RESULT=1
	    fi
	done

	if [ $RESULT -ne 0 ]; then
	    diskmount_umount "$MNT_MD5"
	    return 5
	fi

    fi

	# RTS向けの時に必要なRAID情報を設定する(かつ2発モードでない)
	if [ "$RTSFLAG" = 'ON' -a "$TWOHD" = '' ]; then
		echo "encrypt_raid6" > "$MNT_MD5/conf/raid/$SATARAID1"
	fi

    # 後始末
    diskmount_umount "$MNT_MD5"

    return 0
}


###############################################################################
# md6/md13 を初期化する
###############################################################################
__resetscript_reset_md6_md13() {
    local FUNC="__resetscript_reset_md6_md13"
    local FILE="$FOLDER_RESETSRC/md6.tgz"

    # 上書き用データはあるか
    if [ ! -e "$FILE" ]; then
	# データがない
	echo "*** $FUNC: ERROR: NO DATA ***"
	return 1
    fi

    # 0 クリアする
    if [ "$METHOD" = 'FULL' ]; then
	if ! __resetscript_sanitize_md6; then
	    echo "*** $FUNC: ERROR: FAILED TO SANITIZE DISK $DISK_MD6 ***"
	    return 2
	fi
    fi


    # RAID 構築(md6)
    for DISK in "$DISK_SATA1" "$DISK_SATA2" "$DISK_SATA3" "$DISK_SATA4"; do
	[ "$TWOHD" != '' -a "$DISK" = '' ] && break
	/sbin/mdadm \
	    --zero-superblock \
	    --force \
	    "/dev/${DISK}6" \
	    1>/dev/null 2>/dev/null
	/bin/dd \
	    if=/dev/zero \
	    of="/dev/${DISK}6" \
	    bs=1M \
	    count=4 \
	    1>/dev/null 2>/dev/null
    done

    ## RAID6 を組む(4発版)
    if [ "$TWOHD" = '' ]; then
	if [ "$METHOD" = 'FULL' ]; then
	    if ! /sbin/mdadm \
		--create "${DISK_MD6}" \
		--force \
		--assume-clean \
		--run \
		--level=raid6 \
		--raid-devices=4 \
		"/dev/${DISK_SATA1}6" \
		"/dev/${DISK_SATA2}6" \
		"/dev/${DISK_SATA3}6" \
		"/dev/${DISK_SATA4}6" \
		1>/dev/null 2>/dev/null; then
		echo "*** $FUNC: ERROR: FAILED TO CREATE RAID $DISK_MD6 ***"
		return 3
	    fi
	else
	    if ! /sbin/mdadm \
		--create "${DISK_MD6}" \
		--force \
		--run \
		--level=raid6 \
		--raid-devices=4 \
		"/dev/${DISK_SATA1}6" \
		"/dev/${DISK_SATA2}6" \
		"/dev/${DISK_SATA3}6" \
		"/dev/${DISK_SATA4}6" \
		1>/dev/null 2>/dev/null; then
		echo "*** $FUNC: ERROR: FAILED TO CREATE RAID $DISK_MD6 ***"
		return 3
	    fi
	fi

    ## RAID1 を組む(2発版)
    else
	if [ "$METHOD" = 'FULL' ]; then
	    if ! /sbin/mdadm \
		--create "${DISK_MD6}" \
		--force \
		--assume-clean \
		--run \
		--level=raid1 \
		--raid-devices=2 \
		"/dev/${DISK_SATA1}6" \
		"/dev/${DISK_SATA2}6" \
		1>/dev/null 2>/dev/null; then
		echo "*** $FUNC: ERROR: FAILED TO CREATE RAID $DISK_MD6 ***"
		return 3
	    fi
	else
	    if ! /sbin/mdadm \
		--create "${DISK_MD6}" \
		--force \
		--run \
		--level=raid1 \
		--raid-devices=2 \
		"/dev/${DISK_SATA1}6" \
		"/dev/${DISK_SATA2}6" \
		1>/dev/null 2>/dev/null; then
		echo "*** $FUNC: ERROR: FAILED TO CREATE RAID $DISK_MD6 ***"
		return 3
	    fi
	fi
    fi

    # RAID 構築(md13)
    /sbin/mdadm \
	--zero-superblock \
	--force \
	"$DISK_MD6" \
	1>/dev/null 2>/dev/null
    dd \
	if=/dev/zero \
	of="$DISK_MD6" \
	bs=1k \
	count=4 \
	1>/dev/null 2>/dev/null
    if ! echo y | /sbin/mdadm \
	--create "$DISK_MD13" \
	--force \
	--run \
	--level=raid1 \
	--raid-devices=2 \
	"$DISK_MD6" missing 1>/dev/null 2>/dev/null; then
	sync; sync; sync
	/sbin/mdadm --stop "$DISK_MD6" 1>/dev/null 2>/dev/null
	echo "*** $FUNC: ERROR: FAILED TO CREATE RAID ${DISK_MD}13 ***"
	return 4
    fi

	# 暗号化する
	TARGET_DISK="$DISK_MD13"
	if [ "$RTSFLAG" = 'ON' ]; then
		echo "--- $FUNC: ENCRYPT SETTING START ---"
		TARGET_DISK="$ENC_DEV_ROOT"/"$ENC_DEV_MD13"
#		cp $MNT_USB/$FOLDER_FIRMWARE/encryptctrl.sh /sbin/
#		cp $MNT_USB/$FOLDER_FIRMWARE/mkfs.msdos /sbin/
#		cp $MNT_USB/$FOLDER_FIRMWARE/enckey /sbin/
#		cp $MNT_USB/$FOLDER_FIRMWARE/cryptsetup.sh /sbin/
#		cp $MNT_USB/$FOLDER_FIRMWARE/dmsetup /sbin/
#		cp $MNT_USB/$FOLDER_FIRMWARE/blockdev /sbin/
		local key_dev_mnt_path="/mnt/usb1"
		local key_data_path="/tmp/data"
		/sbin/encryptctrl.sh --keydevinit "$DISK_USB1" "$key_dev_mnt_path" || return 8
		/sbin/encryptctrl.sh --keycreate "$DISK_USB1" "$key_dev_mnt_path" || return 8
		/sbin/encryptctrl.sh --keyextract "$DISK_USB1" "$key_dev_mnt_path" "$key_data_path" || return 8
		/sbin/encryptctrl.sh --cryptsetup "md13" "$key_data_path" || return 8
		echo "--- $FUNC: ENCRYPT SETTING END ---"
	fi

    # フォーマット
#    if ! /sbin/mkfs.xfs -f -q "$DISK_MD13"; then
    if ! /sbin/mkfs.xfs -f -q "$TARGET_DISK"; then
	/bin/sync
	/bin/sync
	/bin/sync
	/sbin/mdadm --stop "$DISK_MD13" 1>/dev/null 2>/dev/null
	/bin/sync
	/bin/sync
	/bin/sync
	/sbin/mdadm --stop "$DISK_MD6" 1>/dev/null 2>/dev/null
	echo "*** $FUNC: ERROR: FAILED TO FORMAT VOLUME ***"
	return 5
    fi

    # マウント
#    if ! diskmount_mount rw "$DISK_MD13" "$MNT_MD6"; then
    if ! diskmount_mount rw "$TARGET_DISK" "$MNT_MD6"; then
	# マウントできない
	## 暗号化を停止
	if [ "$RTSFLAG" = 'ON' ]; then
		echo "--- $FUNC: STOP ENCRYPT $ENC_DEV_MD13 ---"
		/sbin/encryptctrl.sh --stop "$ENC_DEV_MD13"
	fi
	/bin/sync
	/bin/sync
	/bin/sync
	/sbin/mdadm --stop "$DISK_MD13" 1>/dev/null 2>/dev/null
	/bin/sync
	/bin/sync
	/bin/sync
	/sbin/mdadm --stop "$DISK_MD6" 1>/dev/null 2>/dev/null
	echo "*** $FUNC: ERROR: FAILED TO MOUNT VOLUME ***"
	return 6
    fi

    # コピー
    if ! tar zxf "$FILE" -C "$MNT_MD6"; then
	# コピーできない
	## 暗号化を停止
	if [ "$RTSFLAG" = 'ON' ]; then
		echo "--- $FUNC: STOP ENCRYPT $ENC_DEV_MD13 ---"
		/sbin/encryptctrl.sh --stop "$ENC_DEV_MD13"
	fi
	diskmount_umount "${MNT_MD6}"
	/bin/sync
	/bin/sync
	/bin/sync
	/sbin/mdadm --stop ${DISK_MD13} 1>/dev/null 2>/dev/null
	/bin/sync
	/bin/sync
	/bin/sync
	/sbin/mdadm --stop ${DISK_MD6} 1>/dev/null 2>/dev/null
	echo "*** ${FUNC}: ERROR: FAILED TO COPY FILES ***"
	return 7
    fi

    # 後始末
    diskmount_umount "${MNT_MD6}"
	## 暗号化を停止
	if [ "$RTSFLAG" = 'ON' ]; then
		echo "--- $FUNC: STOP ENCRYPT $ENC_DEV_MD13 ---"
		/sbin/encryptctrl.sh --stop "$ENC_DEV_MD13"
	fi
    /bin/sync
    /bin/sync
    /bin/sync
    /sbin/mdadm --stop ${DISK_MD13} 1>/dev/null 2>/dev/null
    /bin/sync
    # md6の停止に失敗することがある為、wait を挿入
    sleep 3
    /bin/sync
    /bin/sync
    /sbin/mdadm --stop ${DISK_MD6} 1>/dev/null 2>/dev/null

    return 0
}



###############################################################################
# 初期化後の処理
###############################################################################
__resetscript_post_reset() {
    local FUNC="__resetscript_post_reset"

    # Intel NIC の WOL を有効にする
    # EEPROMの設定を確認する
    local WOL_EEPROM_ETH1=$(
	/usr/sbin/ethtool -e eth1 \
		2>/dev/null \
	| /bin/grep '0x0040' \
		2>/dev/null \
	| /usr/bin/cut -d" " -f10 \
		2>/dev/null
    )

    echo "--- $FUNC: CHECK WOL EEPROM ---"
    if [ "x${WOL_EEPROM_ETH1}" = "x4d" ]; then
	echo "--- $FUNC: WOL STATUS: ENABLE ---"
    else
	echo "--- $FUNC: WOL EEPROM: DISABLE ---"
	echo "--- $FUNC: ENABLING WOL EEPROM ---"
	/usr/sbin/ethtool -E eth1 magic 0x10d38086 offset 0x49 value 0x4d \
		1>/dev/null 2>/dev/null
    fi

    # REGの設定も確認する
    local WOL_STATUS_ETH1=$(
	/usr/sbin/ethtool eth1 \
		2>/dev/null \
	| /bin/grep '^	Wake-on: ' \
		2>/dev/null \
	| /usr/bin/cut -d: -f2 \
		2>/dev/null
    )

    echo "--- $FUNC: CHECK WOL STATUS ---"
    if [ "x${WOL_STATUS_ETH1}" = "x g" ]; then
	echo "--- $FUNC: WOL STATUS: ENABLE ---"
    else
	echo "--- $FUNC: WOL STATUS: DISABLE ---"
	echo "--- $FUNC: ENABLING WOL ---"
    	/usr/sbin/ethtool -s eth1 wol g \
	    1>/dev/null 2>/dev/null
    fi

    # AC電源復旧時の自動起動設定を無効とする
    local reply=`/usr/local/bin/R8sendmsg -s 'intrp 3 0' -r`
    if [ "x$reply" != 'x0' ]; then
	echo "--- $FUNC: FAIL DISABLE AUTOBOOT CONFIG ---"
	# error return はしない
    fi

    # md5 のマウント
    if ! diskmount_mount rw "${DISK_MD5}" "${MNT_MD5}"; then
	# マウントできない
	echo "*** ${FUNC}: ERROR: FAILED TO MOUNT VOLUME ***"
	return 1
    fi

    # device_uuid があるか
    if [ ! -e "${FOLDER_RESETSRC}/device_uuid" ]; then
	# device_uuid がない(通常ここにはこない)
	diskmount_umount "${MNT_MD5}"
	echo "*** ${FUNC}: ERROR: DEVICE UUID FILE NOT FOUND ***"
	return 2
    fi

    # device_uuid の書き戻し
    if ! /bin/cp -a "${FOLDER_RESETSRC}/device_uuid" "${FOLDER_DEVUUID}/device_uuid"; then
	# 書き戻しできない
	diskmount_umount "${MNT_MD5}"
	echo "*** ${FUNC}: ERROR: FAILED TO RESTORE DEVICE UUID FILE ***"
	return 3
    fi

    # インストールログがあるか
    if [ ! -e "${FOLDER_RESETSRC}/factory_log.tar.gz" ]; then
	# インストールログがない(通常ここにはこない)
	diskmount_umount "${MNT_MD5}"
	echo "*** ${FUNC}: ERROR: INSTALL LOG NOT FOUND ***"
	return 4
    fi

    # インストールログの書き戻し
    if ! tar zxf "${FOLDER_RESETSRC}/factory_log.tar.gz" -C "${MNT_MD5}"; then
	# 書き戻しできない
	diskmount_umount "${MNT_MD5}"
	echo "*** ${FUNC}: ERROR: FAILED TO RESTORE INSTALL LOG ***"
	return 5
    fi

    # 使用済みファイルの削除
    rm -f "${FOLDER_RESETSRC}/factory_log.tar.gz"
    rm -f "${FOLDER_RESETSRC}/device_uuid"

    # 初期化 OK のフラグを立てる
    touch "${FOLDER_RESETSRC}/reset_ok"

    # md5 のアンマウント
    diskmount_umount "${MNT_MD5}"

    # md1 のアンマウント
    diskmount_umount "${MNT_MD1}"

    return 0
}


###############################################################################
# Ver1.02追加項目
# reset.sh（script.sh）が実行可能状態にあるか確認
###############################################################################
__resetscript_is_startcheck() {
    local FUNC="__resetscript_is_startcheck"

    # 領域をマウント
    if ! diskmount_mount ro "$DISK_MD1" "$MNT_MD1"; then
        echo "*** $FUNC: ERROR: FAILED TO MOUNT /dev/md1 ***"
        return 1
    fi

    if [ ! -e "${FOLDER_RESETSRC}/script.sh" ]; then
	# reset.sh（script.sh）がない場合、正規の初期化処理を踏んでいないので
	# エラー停止する。
	# （通常この処理にくることはないはずだが、念のため確認を行う）
	echo "*** ${FUNC}: ERROR: RESET SCRIPT NOT FOUND ***"
	return 1
    fi

    # 領域をアンマウント
    diskmount_umount "$MNT_MD1"

    return 0
}


###############################################################################
# Ver1.02追加項目
# reset.logにエラー停止したときの詳細な情報を残す
###############################################################################
__resetscript_write_resetlog() {
    local FUNC="__resetscript_write_resetlog"
    local ERR_MSG="$1"
    local ERR_CODE="$2"

    # ログ書き込み
    local FILE_RESET_LOG="$FOLDER_FACTLOG/reset.log"

    if ! diskmount_mount rw "$DISK_MD5" "$MNT_MD5"; then
        echo "*** $FUNC: ERROR: FAILED TO MOUNT /dev/md5 ***"
        exit 1
    fi

    if [ -x /bin/touch ]; then
        /bin/touch "$FILE_RESET_LOG"
    fi


    # エラーメッセージをログに残す
    echo "$ERR_MSG"
    echo "$ERR_MSG($ERR_CODE)" >> "$FILE_RESET_LOG"
    diskmount_umount "$MNT_MD5"

    # reset.sh（script.sh）をエラーで抜ける
    exit 1
}


###############################################################################
# メインルーチン
###############################################################################

FAIL_CODE=""

echo
echo "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): START ***"

mddetect_check_md


# Ver1.02追加項目
# reset.sh（script.sh）が本当に実行可能状態にあるか確認
echo
echo "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): RESET SCRIPT START CHECK ***"

__resetscript_is_startcheck

FAIL_CODE="$?"

if [ $FAIL_CODE -ne 0 ]; then
    # reset.sh（script.sh）が実行可能状態はない
    __resetscript_write_resetlog "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): ERROR: CANNOT START RESET SCRIPT ***" "$FAIL_CODE"
fi


# 実行可能環境かの確認
echo
echo "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): INITIAL CHECK ***"
__resetscript_is_executable

FAIL_CODE="$?"

if [ $FAIL_CODE -ne 0 ]; then
    # 実行可能環境ではない
    __resetscript_write_resetlog "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): ERROR: CANNOT EXECUTABLE RESET SCRIPT ***" "$FAIL_CODE"
fi


# 最小LBAのDISKを取得
MODELDISK=$(__resetscript_get_lba_mindisk 0)
if [ "$MODELDISK" = '' ]; then
    # 取得失敗
    echo
    __resetscript_write_resetlog "*** ERROR: FAILED TO GET LBA MINDISK ***"
fi

# 最小LBAのLBAを取得
MODELLBA=$(__resetscript_get_lba_mindisk 1)
if [ "$MODELLBA" = '' ]; then
    # 取得失敗
    echo
    __resetscript_write_resetlog "*** ERROR: FAILED TO GET LBA MINLBA ***"
fi


# factory_log の退避
echo
echo "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): SAVE FACTORY LOG ***"
__resetscript_save_factory_log

FAIL_CODE="$?"

if [ $FAIL_CODE -ne 0 ]; then
    # factory_log を保存できない
    diskmount_umount "${MNT_MD1}"
    __resetscript_write_resetlog "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): ERROR: FAILED TO SAVE FACTORY LOG ***" "$FAIL_CODE"
fi

# device_uuid の退避
echo
echo "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): SAVE DEVICE UUID FILE ***"
__resetscript_save_device_uuid

FAIL_CODE="$?"

if [ $FAIL_CODE -ne 0 ]; then
    # device_uuid を保存できない
    diskmount_umount "${MNT_MD1}"
    __resetscript_write_resetlog "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): ERROR: FAILED TO SAVE DEVICE UUID FILE ***" "$FAIL_CODE"
fi

# 非システムディスクの初期化
__resetscript_reset_partition_disklabel_raid

FAIL_CODE="$?"

if [ $FAIL_CODE -ne 0 ]; then
    # 初期化できない
    diskmount_umount "${MNT_MD1}"
    __resetscript_write_resetlog "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): ERROR: FAILED TO REBUILD NON BOOTABLE DISK ***" "$FAIL_CODE"
fi

# ディスクラベルの更新
echo
echo "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): CHANGE DISK LABEL ***"
for DISK in ${DISK_SATA1} ${DISK_SATA2} ${DISK_SATA3} ${DISK_SATA4}; do
    [ "$TWOHD" != '' -a "$DISK" = '' ] && break
    __resetscript_write_disklabel "${DISK}"

    FAIL_CODE="$?"

    if [ $FAIL_CODE -ne 0 ]; then

	# ディスクラベルを変更できない
	diskmount_umount "${MNT_MD1}"
	__resetscript_write_resetlog "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): ERROR: FAILED TO CHANGE DISK LABEL OF ${DISK} ***" "$FAIL_CODE"
    fi
done

# RTC アラームを無効にする
echo
echo "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): RESET RTC ALARM ***"
__resetscript_reset_alarm

FAIL_CODE="$?"

if [ $FAIL_CODE -ne 0 ]; then
    # RTC アラームを無効にできない
    diskmount_umount "${MNT_MD1}"
    __resetscript_write_resetlog "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): ERROR: FAILED TO RESET RTC ALARM ***" "$FAIL_CODE"
fi

# md1 (上書き)
echo
echo "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): RESET md1 ***"
__resetscript_reset_md1

FAIL_CODE="$?"

if [ $FAIL_CODE -ne 0 ]; then
    # md1 を初期化できない
    diskmount_umount "${MNT_MD1}"
    __resetscript_write_resetlog "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): ERROR: FAILED TO RESET md1 ***" "$FAIL_CODE"
fi

# md2 (フォーマット/インストール)
echo
echo "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): RESET md2 ***"
__resetscript_reset_md2

FAIL_CODE="$?"

if [ $FAIL_CODE -ne 0 ]; then
    # md2 を初期化できない
    diskmount_umount "${MNT_MD1}"
    __resetscript_write_resetlog "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): ERROR: FAILED TO RESET md2 ***" "$FAIL_CODE"
fi

# md5 (フォーマット/インストール)
echo
echo "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): RESET md5 ***"
__resetscript_reset_md5

FAIL_CODE="$?"

if [ $FAIL_CODE -ne 0 ]; then
    # md5 を初期化できない
    diskmount_umount "${MNT_MD1}"
    __resetscript_write_resetlog "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): ERROR: FAILED TO RESET md5 ***" "$FAIL_CODE"
fi

# md6, md13 (RAID 構築/フォーマット/インストール/RAID 停止)
echo
echo "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): RESET md6/md13 ***"
__resetscript_reset_md6_md13

FAIL_CODE="$?"

if [ $FAIL_CODE -ne 0 ]; then
    # md6/md13 を初期化できない
    diskmount_umount "${MNT_MD1}"
    __resetscript_write_resetlog "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): ERROR: FAILED TO RESET md6/md13 ***" "$FAIL_CODE"
fi

# 後始末
echo
echo "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): POST RESET ***"
__resetscript_post_reset

FAIL_CODE="$?"

if [ $FAIL_CODE -ne 0 ]; then
    # 後始末に失敗
    diskmount_umount "${MNT_MD1}"
    __resetscript_write_resetlog "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): ERROR: FAILED TO EXECUTE POST RESET ***" "$FAIL_CODE"
fi

echo
echo "*** RESET(CLEAR:${METHOD} RTS:${RTSFLAG}): RESET SUCCESS ***"
exit 0
