#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from iobase.cmdexec import CmdExec, PIPE, NULL
from iobase.xmldata import XmlNode
from ionas.request import Request, Entry, EntryList
from fal_info import FAL_SHARES_PATH, ReqTypes

SHARELIST_NAME = "sharelist"
SHARE_NAME_PATH = SHARELIST_NAME + "/share/name"

HDL_SHARE_CMD = "/usr/local/bin/HDL_share"

SHARE_FMT = """<share>
                    <name>%s</name>
                    <path>%s</path>
                    <shareservice>
                        <amazons3 enable="false"/>
                    </shareservice>
               </share>"""

REMOVABLE_SHARE_LIST = ("usb1", "usb2", "esata1")


def __request(fal, data):
    return Request(fal).request(data)


def _request(cmd, type=None, data=None):
    if type:
        if not data:
            return __request(cmd, Entry(type)).\
                get_entrylist().get_entry()
        if isinstance(data, str):
            return __request(cmd, Entry(type, XmlNode(data))).\
                get_entrylist().get_entry()
        if isinstance(data, XmlNode):
            return __request(cmd, Entry(type, data)).\
                get_entrylist().get_entry()
    else:
        if isinstance(data, EntryList):
            return  __request(cmd, data).\
                get_entrylist()
        if (isinstance(data, Entry) or
            isinstance(data, XmlNode)):
            return  __request(cmd, data).\
                get_entrylist().get_entry()


def _get_already_exist_share_list():
    already_exist_share_list = []

    proc = CmdExec([HDL_SHARE_CMD]).execute(stdout=PIPE, stderr=NULL)

    for line in proc.stdout.readlines():
        if line.startswith("\t"):
            continue
        already_exist_share_list.append(line.strip())

    proc.wait()

    return already_exist_share_list


def _fal_add_share(share_list):
    fal_share_conf = _request(FAL_SHARES_PATH, ReqTypes.LOAD).get()
    fal_share_list = fal_share_conf.get_value_list(SHARE_NAME_PATH)

    not_exist_share_list = set(share_list) - set(fal_share_list)

    for name in not_exist_share_list:
        if name in REMOVABLE_SHARE_LIST:
            path = "/mnt/" + name
        else:
            path = "/mnt/sataraid1/share/" + name

        fal_share_conf.get_node(SHARELIST_NAME).append_child(
                    XmlNode(SHARE_FMT % (name, path)))

    _request(FAL_SHARES_PATH, ReqTypes.SAVE, fal_share_conf)


if __name__ == "__main__":
    _fal_add_share(_get_already_exist_share_list())
